@@ -111,7 +111,8 @@ is written to the referenced string or file handle.
 package Mail::DKIM::Verifier;
 use base "Mail::DKIM::Common";
 use Carp;
-our $VERSION = 0.38;
+our $VERSION = 0.39;
+our $MAX_SIGNATURES_TO_PROCESS = 50;
 
 sub init
 {
@@ -157,6 +158,7 @@ sub handle_header
 		{
 			my $signature = Mail::DKIM::Signature->parse($line);
 			$self->add_signature($signature);
+			$signature->fetch_public_key;
 		};
 		if ($@)
 		{
@@ -176,6 +178,7 @@ sub handle_header
 		{
 			my $signature = Mail::DKIM::DkSignature->parse($line);
 			$self->add_signature($signature);
+			$signature->fetch_public_key;
 		};
 		if ($@)
 		{
@@ -196,6 +199,10 @@ sub add_signature
 	croak "wrong number of arguments" unless (@_ == 1);
 	my ($signature) = @_;
 
+	# ignore signature headers once we've seen 50 or so
+	# this protects against abuse.
+	return if (@{$self->{signatures}} > $MAX_SIGNATURES_TO_PROCESS);
+
 	push @{$self->{signatures}}, $signature;
 
 	unless ($self->check_signature($signature))
@@ -394,44 +401,6 @@ sub finish_header
 		$algorithm->finish_header;
 	}
 
-	# For each parsed signature, check it for validity. If none are valid,
-	# our result is "invalid" and our result detail will be the reason
-	# why the last signature was invalid.
-
-	foreach my $signature (@{$self->{signatures}})
-	{
-		unless (check_signature_identity($signature))
-		{
-			$self->{signature_reject_reason} = "bad identity";
-			$signature->result("invalid",
-				$self->{signature_reject_reason});
-			next;
-		}
-
-		# get public key
-		my $pkey;
-		eval
-		{
-			$pkey = $signature->get_public_key;
-		};
-		if ($@)
-		{
-			my $E = $@;
-			chomp $E;
-			$self->{signature_reject_reason} = "public key: $E";
-			$signature->result("invalid",
-				$self->{signature_reject_reason});
-			next;
-		}
-
-		unless ($self->check_public_key($signature, $pkey))
-		{
-			$signature->result("invalid",
-				$self->{signature_reject_reason});
-			next;
-		}
-	}
-
 	# stop processing signatures that are already known to be invalid
 	@{$self->{algorithms}} = grep
 		{
@@ -448,14 +417,37 @@ sub finish_header
 	}
 }
 
-sub finish_body
+sub _check_and_verify_signature
 {
 	my $self = shift;
+	my ($algorithm) = @_;
 
-	foreach my $algorithm (@{$self->{algorithms}})
-	{
-		# finish canonicalizing
-		$algorithm->finish_body;
+		# check signature
+		my $signature = $algorithm->signature;
+		unless (check_signature_identity($signature))
+		{
+			$self->{signature_reject_reason} = "bad identity";
+			return ("invalid", $self->{signature_reject_reason});
+		}
+
+		# get public key
+		my $pkey;
+		eval
+		{
+			$pkey = $signature->get_public_key;
+		};
+		if ($@)
+		{
+			my $E = $@;
+			chomp $E;
+			$self->{signature_reject_reason} = "public key: $E";
+			return ("invalid", $self->{signature_reject_reason});
+		}
+
+		unless ($self->check_public_key($signature, $pkey))
+		{
+			return ("invalid", $self->{signature_reject_reason});
+		}
 
 		# verify signature
 		my $result;
@@ -481,6 +473,19 @@ sub finish_body
 			$result = "fail";
 			$details = $E;
 		}
+	return ($result, $details);
+}
+
+sub finish_body
+{
+	my $self = shift;
+
+	foreach my $algorithm (@{$self->{algorithms}})
+	{
+		# finish canonicalizing
+		$algorithm->finish_body;
+
+		my ($result, $details) = $self->_check_and_verify_signature($algorithm);
 
 		# save the results of this signature verification
 		$algorithm->{result} = $result;
